# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _hypre
else:
    import _hypre

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _hypre.SWIG_PyInstanceMethod_New
_swig_new_static_method = _hypre.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _hypre.MFEM_VERSION

MFEM_VERSION_STRING = _hypre.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _hypre.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _hypre.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _hypre.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _hypre.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _hypre.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _hypre.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _hypre.MFEM_HYPRE_VERSION

import mfem._par.globals
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.sparsemat
import mfem._par.operators
import mfem._par.matrix
import mfem._par.densemat

def sizeof_HYPRE_Int():
    r"""sizeof_HYPRE_Int() -> int"""
    return _hypre.sizeof_HYPRE_Int()
sizeof_HYPRE_Int = _hypre.sizeof_HYPRE_Int
class HypreParVector(mfem._par.vector.Vector):
    r"""Proxy of C++ mfem::HypreParVector class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreParVector self, MPI_Comm comm, HYPRE_Int glob_size, HYPRE_Int * col) -> HypreParVector
        __init__(HypreParVector self, MPI_Comm comm, HYPRE_Int glob_size, double * _data) -> HypreParVector
        __init__(HypreParVector self, HypreParVector y) -> HypreParVector
        __init__(HypreParVector self, HypreParMatrix A, int transpose=0) -> HypreParVector
        __init__(HypreParVector self, HYPRE_ParVector y) -> HypreParVector
        __init__(HypreParVector self, mfem::ParFiniteElementSpace * pfes) -> HypreParVector
        """

        import numpy as np
        self._linked_array = None          
        if isinstance(args[-1], list):
            v = np.ascontiguousarray(args[-1][0])
            col = np.ascontiguousarray(args[-1][1])
            args = list(args[:-1])
            args.append([v, col])


        _hypre.HypreParVector_swiginit(self, _hypre.new_HypreParVector(*args))

        if isinstance(args[-1], list):
        # in this case, ParVector does not own the object
        # in order to prevent python from freeing the input
        # array, object is kept in ParVector
           self._linked_array = args[-1]




    def GetComm(self):
        r"""GetComm(HypreParVector self) -> MPI_Comm"""
        return _hypre.HypreParVector_GetComm(self)
    GetComm = _swig_new_instance_method(_hypre.HypreParVector_GetComm)

    def Partitioning(self):
        r"""Partitioning(HypreParVector self) -> HYPRE_Int *"""
        return _hypre.HypreParVector_Partitioning(self)
    Partitioning = _swig_new_instance_method(_hypre.HypreParVector_Partitioning)

    def GlobalSize(self):
        r"""GlobalSize(HypreParVector self) -> HYPRE_Int"""
        return _hypre.HypreParVector_GlobalSize(self)
    GlobalSize = _swig_new_instance_method(_hypre.HypreParVector_GlobalSize)

    def StealParVector(self):
        r"""StealParVector(HypreParVector self) -> hypre_ParVector *"""
        return _hypre.HypreParVector_StealParVector(self)
    StealParVector = _swig_new_instance_method(_hypre.HypreParVector_StealParVector)

    def SetOwnership(self, own):
        r"""SetOwnership(HypreParVector self, int own)"""
        return _hypre.HypreParVector_SetOwnership(self, own)
    SetOwnership = _swig_new_instance_method(_hypre.HypreParVector_SetOwnership)

    def GetOwnership(self):
        r"""GetOwnership(HypreParVector self) -> int"""
        return _hypre.HypreParVector_GetOwnership(self)
    GetOwnership = _swig_new_instance_method(_hypre.HypreParVector_GetOwnership)

    def GlobalVector(self):
        r"""GlobalVector(HypreParVector self) -> Vector"""
        return _hypre.HypreParVector_GlobalVector(self)
    GlobalVector = _swig_new_instance_method(_hypre.HypreParVector_GlobalVector)

    def SetData(self, _data):
        r"""SetData(HypreParVector self, double * _data)"""
        return _hypre.HypreParVector_SetData(self, _data)
    SetData = _swig_new_instance_method(_hypre.HypreParVector_SetData)

    def Randomize(self, seed):
        r"""Randomize(HypreParVector self, HYPRE_Int seed) -> HYPRE_Int"""
        return _hypre.HypreParVector_Randomize(self, seed)
    Randomize = _swig_new_instance_method(_hypre.HypreParVector_Randomize)

    def Print(self, fname):
        r"""Print(HypreParVector self, char const * fname)"""
        return _hypre.HypreParVector_Print(self, fname)
    Print = _swig_new_instance_method(_hypre.HypreParVector_Print)
    __swig_destroy__ = _hypre.delete_HypreParVector

    def GetPartitioningArray(self):
        r"""GetPartitioningArray(HypreParVector self) -> PyObject *"""
        return _hypre.HypreParVector_GetPartitioningArray(self)
    GetPartitioningArray = _swig_new_instance_method(_hypre.HypreParVector_GetPartitioningArray)

# Register HypreParVector in _hypre:
_hypre.HypreParVector_swigregister(HypreParVector)


def ParNormlp(vec, p, comm):
    r"""ParNormlp(Vector vec, double p, MPI_Comm comm) -> double"""
    return _hypre.ParNormlp(vec, p, comm)
ParNormlp = _hypre.ParNormlp
class HypreParMatrix(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::HypreParMatrix class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreParMatrix self) -> HypreParMatrix
        __init__(HypreParMatrix self, hypre_ParCSRMatrix * a, bool owner=True) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int glob_size, HYPRE_Int * row_starts, SparseMatrix diag) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int global_num_rows, HYPRE_Int global_num_cols, HYPRE_Int * row_starts, HYPRE_Int * col_starts, SparseMatrix diag) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int global_num_rows, HYPRE_Int global_num_cols, HYPRE_Int * row_starts, HYPRE_Int * col_starts, SparseMatrix diag, SparseMatrix offd, HYPRE_Int * cmap) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int global_num_rows, HYPRE_Int global_num_cols, HYPRE_Int * row_starts, HYPRE_Int * col_starts, HYPRE_Int * diag_i, HYPRE_Int * diag_j, double * diag_data, HYPRE_Int * offd_i, HYPRE_Int * offd_j, double * offd_data, HYPRE_Int offd_num_cols, HYPRE_Int * offd_col_map) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int * row_starts, HYPRE_Int * col_starts, SparseMatrix a) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, HYPRE_Int global_num_rows, HYPRE_Int global_num_cols, HYPRE_Int * row_starts, HYPRE_Int * col_starts, mfem::Table * diag) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, int id, int np, HYPRE_Int * row, HYPRE_Int * col, HYPRE_Int * i_diag, HYPRE_Int * j_diag, HYPRE_Int * i_offd, HYPRE_Int * j_offd, HYPRE_Int * cmap, HYPRE_Int cmap_size) -> HypreParMatrix
        __init__(HypreParMatrix self, MPI_Comm comm, int nrows, HYPRE_Int glob_nrows, HYPRE_Int glob_ncols, int * I) -> HypreParMatrix
        __init__(HypreParMatrix self, HypreParMatrix P) -> HypreParMatrix
        """
        _hypre.HypreParMatrix_swiginit(self, _hypre.new_HypreParMatrix(*args))

    def MakeRef(self, master):
        r"""MakeRef(HypreParMatrix self, HypreParMatrix master)"""
        return _hypre.HypreParMatrix_MakeRef(self, master)
    MakeRef = _swig_new_instance_method(_hypre.HypreParMatrix_MakeRef)

    def GetComm(self):
        r"""GetComm(HypreParMatrix self) -> MPI_Comm"""
        return _hypre.HypreParMatrix_GetComm(self)
    GetComm = _swig_new_instance_method(_hypre.HypreParMatrix_GetComm)

    def StealData(self):
        r"""StealData(HypreParMatrix self) -> hypre_ParCSRMatrix *"""
        return _hypre.HypreParMatrix_StealData(self)
    StealData = _swig_new_instance_method(_hypre.HypreParMatrix_StealData)

    def SetOwnerFlags(self, diag, offd, colmap):
        r"""SetOwnerFlags(HypreParMatrix self, signed char diag, signed char offd, signed char colmap)"""
        return _hypre.HypreParMatrix_SetOwnerFlags(self, diag, offd, colmap)
    SetOwnerFlags = _swig_new_instance_method(_hypre.HypreParMatrix_SetOwnerFlags)

    def OwnsDiag(self):
        r"""OwnsDiag(HypreParMatrix self) -> signed char"""
        return _hypre.HypreParMatrix_OwnsDiag(self)
    OwnsDiag = _swig_new_instance_method(_hypre.HypreParMatrix_OwnsDiag)

    def OwnsOffd(self):
        r"""OwnsOffd(HypreParMatrix self) -> signed char"""
        return _hypre.HypreParMatrix_OwnsOffd(self)
    OwnsOffd = _swig_new_instance_method(_hypre.HypreParMatrix_OwnsOffd)

    def OwnsColMap(self):
        r"""OwnsColMap(HypreParMatrix self) -> signed char"""
        return _hypre.HypreParMatrix_OwnsColMap(self)
    OwnsColMap = _swig_new_instance_method(_hypre.HypreParMatrix_OwnsColMap)

    def CopyRowStarts(self):
        r"""CopyRowStarts(HypreParMatrix self)"""
        return _hypre.HypreParMatrix_CopyRowStarts(self)
    CopyRowStarts = _swig_new_instance_method(_hypre.HypreParMatrix_CopyRowStarts)

    def CopyColStarts(self):
        r"""CopyColStarts(HypreParMatrix self)"""
        return _hypre.HypreParMatrix_CopyColStarts(self)
    CopyColStarts = _swig_new_instance_method(_hypre.HypreParMatrix_CopyColStarts)

    def NNZ(self):
        r"""NNZ(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_NNZ(self)
    NNZ = _swig_new_instance_method(_hypre.HypreParMatrix_NNZ)

    def RowPart(self, *args):
        r"""
        RowPart(HypreParMatrix self) -> HYPRE_Int
        RowPart(HypreParMatrix self) -> HYPRE_Int const *
        """
        return _hypre.HypreParMatrix_RowPart(self, *args)
    RowPart = _swig_new_instance_method(_hypre.HypreParMatrix_RowPart)

    def ColPart(self, *args):
        r"""
        ColPart(HypreParMatrix self) -> HYPRE_Int
        ColPart(HypreParMatrix self) -> HYPRE_Int const *
        """
        return _hypre.HypreParMatrix_ColPart(self, *args)
    ColPart = _swig_new_instance_method(_hypre.HypreParMatrix_ColPart)

    def M(self):
        r"""M(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_M(self)
    M = _swig_new_instance_method(_hypre.HypreParMatrix_M)

    def N(self):
        r"""N(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_N(self)
    N = _swig_new_instance_method(_hypre.HypreParMatrix_N)

    def GetDiag(self, *args):
        r"""
        GetDiag(HypreParMatrix self, Vector diag)
        GetDiag(HypreParMatrix self, SparseMatrix diag)
        """
        return _hypre.HypreParMatrix_GetDiag(self, *args)
    GetDiag = _swig_new_instance_method(_hypre.HypreParMatrix_GetDiag)

    def GetOffd(self, offd, cmap):
        r"""GetOffd(HypreParMatrix self, SparseMatrix offd, HYPRE_Int *& cmap)"""
        return _hypre.HypreParMatrix_GetOffd(self, offd, cmap)
    GetOffd = _swig_new_instance_method(_hypre.HypreParMatrix_GetOffd)

    def GetBlocks(self, blocks, interleaved_rows=False, interleaved_cols=False):
        r"""GetBlocks(HypreParMatrix self, mfem::Array2D< mfem::HypreParMatrix * > & blocks, bool interleaved_rows=False, bool interleaved_cols=False)"""
        return _hypre.HypreParMatrix_GetBlocks(self, blocks, interleaved_rows, interleaved_cols)
    GetBlocks = _swig_new_instance_method(_hypre.HypreParMatrix_GetBlocks)

    def Transpose(self):
        r"""Transpose(HypreParMatrix self) -> HypreParMatrix"""
        return _hypre.HypreParMatrix_Transpose(self)
    Transpose = _swig_new_instance_method(_hypre.HypreParMatrix_Transpose)

    def GetNumRows(self):
        r"""GetNumRows(HypreParMatrix self) -> int"""
        return _hypre.HypreParMatrix_GetNumRows(self)
    GetNumRows = _swig_new_instance_method(_hypre.HypreParMatrix_GetNumRows)

    def GetNumCols(self):
        r"""GetNumCols(HypreParMatrix self) -> int"""
        return _hypre.HypreParMatrix_GetNumCols(self)
    GetNumCols = _swig_new_instance_method(_hypre.HypreParMatrix_GetNumCols)

    def GetGlobalNumRows(self):
        r"""GetGlobalNumRows(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_GetGlobalNumRows(self)
    GetGlobalNumRows = _swig_new_instance_method(_hypre.HypreParMatrix_GetGlobalNumRows)

    def GetGlobalNumCols(self):
        r"""GetGlobalNumCols(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_GetGlobalNumCols(self)
    GetGlobalNumCols = _swig_new_instance_method(_hypre.HypreParMatrix_GetGlobalNumCols)

    def GetRowStarts(self):
        r"""GetRowStarts(HypreParMatrix self) -> HYPRE_Int *"""
        return _hypre.HypreParMatrix_GetRowStarts(self)
    GetRowStarts = _swig_new_instance_method(_hypre.HypreParMatrix_GetRowStarts)

    def GetColStarts(self):
        r"""GetColStarts(HypreParMatrix self) -> HYPRE_Int *"""
        return _hypre.HypreParMatrix_GetColStarts(self)
    GetColStarts = _swig_new_instance_method(_hypre.HypreParMatrix_GetColStarts)

    def Mult(self, *args):
        r"""
        Mult(HypreParMatrix self, HypreParVector x, HypreParVector y, double alpha=1.0, double beta=0.0) -> HYPRE_Int
        Mult(HypreParMatrix self, HYPRE_ParVector x, HYPRE_ParVector y, double alpha=1.0, double beta=0.0) -> HYPRE_Int
        Mult(HypreParMatrix self, double a, Vector x, double b, Vector y)
        Mult(HypreParMatrix self, Vector x, Vector y)
        """
        return _hypre.HypreParMatrix_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HypreParMatrix_Mult)

    def MultTranspose(self, *args):
        r"""
        MultTranspose(HypreParMatrix self, HypreParVector x, HypreParVector y, double alpha=1.0, double beta=0.0) -> HYPRE_Int
        MultTranspose(HypreParMatrix self, double a, Vector x, double b, Vector y)
        MultTranspose(HypreParMatrix self, Vector x, Vector y)
        """
        return _hypre.HypreParMatrix_MultTranspose(self, *args)
    MultTranspose = _swig_new_instance_method(_hypre.HypreParMatrix_MultTranspose)

    def AbsMult(self, a, x, b, y):
        r"""AbsMult(HypreParMatrix self, double a, Vector x, double b, Vector y)"""
        return _hypre.HypreParMatrix_AbsMult(self, a, x, b, y)
    AbsMult = _swig_new_instance_method(_hypre.HypreParMatrix_AbsMult)

    def AbsMultTranspose(self, a, x, b, y):
        r"""AbsMultTranspose(HypreParMatrix self, double a, Vector x, double b, Vector y)"""
        return _hypre.HypreParMatrix_AbsMultTranspose(self, a, x, b, y)
    AbsMultTranspose = _swig_new_instance_method(_hypre.HypreParMatrix_AbsMultTranspose)

    def BooleanMult(self, alpha, x, beta, y):
        r"""BooleanMult(HypreParMatrix self, int alpha, int const * x, int beta, int * y)"""
        return _hypre.HypreParMatrix_BooleanMult(self, alpha, x, beta, y)
    BooleanMult = _swig_new_instance_method(_hypre.HypreParMatrix_BooleanMult)

    def BooleanMultTranspose(self, alpha, x, beta, y):
        r"""BooleanMultTranspose(HypreParMatrix self, int alpha, int const * x, int beta, int * y)"""
        return _hypre.HypreParMatrix_BooleanMultTranspose(self, alpha, x, beta, y)
    BooleanMultTranspose = _swig_new_instance_method(_hypre.HypreParMatrix_BooleanMultTranspose)

    def __iadd__(self, B):
        r"""__iadd__(HypreParMatrix self, HypreParMatrix B) -> HypreParMatrix"""
        return _hypre.HypreParMatrix___iadd__(self, B)
    __iadd__ = _swig_new_instance_method(_hypre.HypreParMatrix___iadd__)

    def Add(self, beta, B):
        r"""Add(HypreParMatrix self, double const beta, HypreParMatrix B) -> HypreParMatrix"""
        return _hypre.HypreParMatrix_Add(self, beta, B)
    Add = _swig_new_instance_method(_hypre.HypreParMatrix_Add)

    def LeftDiagMult(self, D, row_starts=None):
        r"""LeftDiagMult(HypreParMatrix self, SparseMatrix D, HYPRE_Int * row_starts=None) -> HypreParMatrix"""
        return _hypre.HypreParMatrix_LeftDiagMult(self, D, row_starts)
    LeftDiagMult = _swig_new_instance_method(_hypre.HypreParMatrix_LeftDiagMult)

    def ScaleRows(self, s):
        r"""ScaleRows(HypreParMatrix self, Vector s)"""
        return _hypre.HypreParMatrix_ScaleRows(self, s)
    ScaleRows = _swig_new_instance_method(_hypre.HypreParMatrix_ScaleRows)

    def InvScaleRows(self, s):
        r"""InvScaleRows(HypreParMatrix self, Vector s)"""
        return _hypre.HypreParMatrix_InvScaleRows(self, s)
    InvScaleRows = _swig_new_instance_method(_hypre.HypreParMatrix_InvScaleRows)

    def __imul__(self, s):
        r"""__imul__(HypreParMatrix self, double s)"""
        val = _hypre.HypreParMatrix___imul__(self, s)

        #    val.thisown = 0
        return self


        return val


    def Threshold(self, threshold=0.0):
        r"""Threshold(HypreParMatrix self, double threshold=0.0)"""
        return _hypre.HypreParMatrix_Threshold(self, threshold)
    Threshold = _swig_new_instance_method(_hypre.HypreParMatrix_Threshold)

    def EliminateZeroRows(self):
        r"""EliminateZeroRows(HypreParMatrix self)"""
        return _hypre.HypreParMatrix_EliminateZeroRows(self)
    EliminateZeroRows = _swig_new_instance_method(_hypre.HypreParMatrix_EliminateZeroRows)

    def EliminateRowsCols(self, *args):
        r"""
        EliminateRowsCols(HypreParMatrix self, intArray rows_cols, HypreParVector X, HypreParVector B)
        EliminateRowsCols(HypreParMatrix self, intArray rows_cols) -> HypreParMatrix
        """
        return _hypre.HypreParMatrix_EliminateRowsCols(self, *args)
    EliminateRowsCols = _swig_new_instance_method(_hypre.HypreParMatrix_EliminateRowsCols)

    def EliminateCols(self, cols):
        r"""EliminateCols(HypreParMatrix self, intArray cols) -> HypreParMatrix"""
        return _hypre.HypreParMatrix_EliminateCols(self, cols)
    EliminateCols = _swig_new_instance_method(_hypre.HypreParMatrix_EliminateCols)

    def EliminateRows(self, rows):
        r"""EliminateRows(HypreParMatrix self, intArray rows)"""
        return _hypre.HypreParMatrix_EliminateRows(self, rows)
    EliminateRows = _swig_new_instance_method(_hypre.HypreParMatrix_EliminateRows)

    def Print(self, fname, offi=0, offj=0):
        r"""Print(HypreParMatrix self, char const * fname, HYPRE_Int offi=0, HYPRE_Int offj=0)"""
        return _hypre.HypreParMatrix_Print(self, fname, offi, offj)
    Print = _swig_new_instance_method(_hypre.HypreParMatrix_Print)

    def Read(self, comm, fname):
        r"""Read(HypreParMatrix self, MPI_Comm comm, char const * fname)"""
        return _hypre.HypreParMatrix_Read(self, comm, fname)
    Read = _swig_new_instance_method(_hypre.HypreParMatrix_Read)

    def Read_IJMatrix(self, comm, fname):
        r"""Read_IJMatrix(HypreParMatrix self, MPI_Comm comm, char const * fname)"""
        return _hypre.HypreParMatrix_Read_IJMatrix(self, comm, fname)
    Read_IJMatrix = _swig_new_instance_method(_hypre.HypreParMatrix_Read_IJMatrix)
    __swig_destroy__ = _hypre.delete_HypreParMatrix

    def GetType(self):
        r"""GetType(HypreParMatrix self) -> mfem::Operator::Type"""
        return _hypre.HypreParMatrix_GetType(self)
    GetType = _swig_new_instance_method(_hypre.HypreParMatrix_GetType)

    def GetRowPartArray(self):
        r"""GetRowPartArray(HypreParMatrix self) -> PyObject *"""
        return _hypre.HypreParMatrix_GetRowPartArray(self)
    GetRowPartArray = _swig_new_instance_method(_hypre.HypreParMatrix_GetRowPartArray)

    def GetColPartArray(self):
        r"""GetColPartArray(HypreParMatrix self) -> PyObject *"""
        return _hypre.HypreParMatrix_GetColPartArray(self)
    GetColPartArray = _swig_new_instance_method(_hypre.HypreParMatrix_GetColPartArray)

    def get_local_nnz(self):
        r"""get_local_nnz(HypreParMatrix self) -> HYPRE_Int"""
        return _hypre.HypreParMatrix_get_local_nnz(self)
    get_local_nnz = _swig_new_instance_method(_hypre.HypreParMatrix_get_local_nnz)

    def get_local_true_nnz(self):
        r"""get_local_true_nnz(HypreParMatrix self) -> PyObject *"""
        return _hypre.HypreParMatrix_get_local_true_nnz(self)
    get_local_true_nnz = _swig_new_instance_method(_hypre.HypreParMatrix_get_local_true_nnz)

    def GetCooDataArray(self, base_i=0, base_j=0):
        r"""GetCooDataArray(HypreParMatrix self, HYPRE_Int const base_i=0, HYPRE_Int const base_j=0) -> PyObject *"""
        return _hypre.HypreParMatrix_GetCooDataArray(self, base_i, base_j)
    GetCooDataArray = _swig_new_instance_method(_hypre.HypreParMatrix_GetCooDataArray)

    def PrintCommPkg(self, *args):
        r"""
        PrintCommPkg(HypreParMatrix self, std::ostream & out=out)
        PrintCommPkg(HypreParMatrix self, char const * file, int precision=8)
        """
        return _hypre.HypreParMatrix_PrintCommPkg(self, *args)
    PrintCommPkg = _swig_new_instance_method(_hypre.HypreParMatrix_PrintCommPkg)

# Register HypreParMatrix in _hypre:
_hypre.HypreParMatrix_swigregister(HypreParMatrix)


def ParMult(A, B, own_matrix=False):
    r"""ParMult(HypreParMatrix A, HypreParMatrix B, bool own_matrix=False) -> HypreParMatrix"""
    return _hypre.ParMult(A, B, own_matrix)
ParMult = _hypre.ParMult

def ParAdd(A, B):
    r"""ParAdd(HypreParMatrix A, HypreParMatrix B) -> HypreParMatrix"""
    return _hypre.ParAdd(A, B)
ParAdd = _hypre.ParAdd

def HypreParMatrixFromBlocks(blocks, blockCoeff=None):
    r"""HypreParMatrixFromBlocks(mfem::Array2D< mfem::HypreParMatrix * > & blocks, mfem::Array2D< double > * blockCoeff=None) -> HypreParMatrix"""
    return _hypre.HypreParMatrixFromBlocks(blocks, blockCoeff)
HypreParMatrixFromBlocks = _hypre.HypreParMatrixFromBlocks

def EliminateBC(A, Ae, ess_dof_list, X, B):
    r"""EliminateBC(HypreParMatrix A, HypreParMatrix Ae, intArray ess_dof_list, Vector X, Vector B)"""
    return _hypre.EliminateBC(A, Ae, ess_dof_list, X, B)
EliminateBC = _hypre.EliminateBC
class HypreSmoother(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::HypreSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Jacobi = _hypre.HypreSmoother_Jacobi
    
    l1Jacobi = _hypre.HypreSmoother_l1Jacobi
    
    l1GS = _hypre.HypreSmoother_l1GS
    
    l1GStr = _hypre.HypreSmoother_l1GStr
    
    lumpedJacobi = _hypre.HypreSmoother_lumpedJacobi
    
    GS = _hypre.HypreSmoother_GS
    
    Chebyshev = _hypre.HypreSmoother_Chebyshev
    
    Taubin = _hypre.HypreSmoother_Taubin
    
    FIR = _hypre.HypreSmoother_FIR
    

    def __init__(self, *args):
        r"""
        __init__(HypreSmoother self) -> HypreSmoother
        __init__(HypreSmoother self, HypreParMatrix _A, int type=l1GS, int relax_times=1, double relax_weight=1.0, double omega=1.0, int poly_order=2, double poly_fraction=.3, int eig_est_cg_iter=10) -> HypreSmoother
        """
        _hypre.HypreSmoother_swiginit(self, _hypre.new_HypreSmoother(*args))

    def SetType(self, type, relax_times=1):
        r"""SetType(HypreSmoother self, mfem::HypreSmoother::Type type, int relax_times=1)"""
        return _hypre.HypreSmoother_SetType(self, type, relax_times)
    SetType = _swig_new_instance_method(_hypre.HypreSmoother_SetType)

    def SetSOROptions(self, relax_weight, omega):
        r"""SetSOROptions(HypreSmoother self, double relax_weight, double omega)"""
        return _hypre.HypreSmoother_SetSOROptions(self, relax_weight, omega)
    SetSOROptions = _swig_new_instance_method(_hypre.HypreSmoother_SetSOROptions)

    def SetPolyOptions(self, poly_order, poly_fraction, eig_est_cg_iter=10):
        r"""SetPolyOptions(HypreSmoother self, int poly_order, double poly_fraction, int eig_est_cg_iter=10)"""
        return _hypre.HypreSmoother_SetPolyOptions(self, poly_order, poly_fraction, eig_est_cg_iter)
    SetPolyOptions = _swig_new_instance_method(_hypre.HypreSmoother_SetPolyOptions)

    def SetTaubinOptions(self, _lambda, mu, iter):
        r"""SetTaubinOptions(HypreSmoother self, double _lambda, double mu, int iter)"""
        return _hypre.HypreSmoother_SetTaubinOptions(self, _lambda, mu, iter)
    SetTaubinOptions = _swig_new_instance_method(_hypre.HypreSmoother_SetTaubinOptions)

    def SetWindowByName(self, window_name):
        r"""SetWindowByName(HypreSmoother self, char const * window_name)"""
        return _hypre.HypreSmoother_SetWindowByName(self, window_name)
    SetWindowByName = _swig_new_instance_method(_hypre.HypreSmoother_SetWindowByName)

    def SetWindowParameters(self, a, b, c):
        r"""SetWindowParameters(HypreSmoother self, double a, double b, double c)"""
        return _hypre.HypreSmoother_SetWindowParameters(self, a, b, c)
    SetWindowParameters = _swig_new_instance_method(_hypre.HypreSmoother_SetWindowParameters)

    def SetFIRCoefficients(self, max_eig):
        r"""SetFIRCoefficients(HypreSmoother self, double max_eig)"""
        return _hypre.HypreSmoother_SetFIRCoefficients(self, max_eig)
    SetFIRCoefficients = _swig_new_instance_method(_hypre.HypreSmoother_SetFIRCoefficients)

    def SetPositiveDiagonal(self, pos=True):
        r"""SetPositiveDiagonal(HypreSmoother self, bool pos=True)"""
        return _hypre.HypreSmoother_SetPositiveDiagonal(self, pos)
    SetPositiveDiagonal = _swig_new_instance_method(_hypre.HypreSmoother_SetPositiveDiagonal)

    def SetOperator(self, op):
        r"""SetOperator(HypreSmoother self, Operator op)"""
        return _hypre.HypreSmoother_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreSmoother_SetOperator)

    def Mult(self, *args):
        r"""
        Mult(HypreSmoother self, HypreParVector b, HypreParVector x)
        Mult(HypreSmoother self, Vector b, Vector x)
        """
        return _hypre.HypreSmoother_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HypreSmoother_Mult)
    __swig_destroy__ = _hypre.delete_HypreSmoother

# Register HypreSmoother in _hypre:
_hypre.HypreSmoother_swigregister(HypreSmoother)

class HypreSolver(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::HypreSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    IGNORE_HYPRE_ERRORS = _hypre.HypreSolver_IGNORE_HYPRE_ERRORS
    
    WARN_HYPRE_ERRORS = _hypre.HypreSolver_WARN_HYPRE_ERRORS
    
    ABORT_HYPRE_ERRORS = _hypre.HypreSolver_ABORT_HYPRE_ERRORS
    

    def SetupFcn(self):
        r"""SetupFcn(HypreSolver self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreSolver_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreSolver_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreSolver self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreSolver_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreSolver_SolveFcn)

    def SetOperator(self, op):
        r"""SetOperator(HypreSolver self, Operator op)"""
        return _hypre.HypreSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreSolver_SetOperator)

    def Mult(self, *args):
        r"""
        Mult(HypreSolver self, HypreParVector b, HypreParVector x)
        Mult(HypreSolver self, Vector b, Vector x)
        """
        return _hypre.HypreSolver_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HypreSolver_Mult)

    def SetErrorMode(self, err_mode):
        r"""SetErrorMode(HypreSolver self, mfem::HypreSolver::ErrorMode err_mode)"""
        return _hypre.HypreSolver_SetErrorMode(self, err_mode)
    SetErrorMode = _swig_new_instance_method(_hypre.HypreSolver_SetErrorMode)
    __swig_destroy__ = _hypre.delete_HypreSolver

# Register HypreSolver in _hypre:
_hypre.HypreSolver_swigregister(HypreSolver)

class HyprePCG(HypreSolver):
    r"""Proxy of C++ mfem::HyprePCG class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HyprePCG self, MPI_Comm comm) -> HyprePCG
        __init__(HyprePCG self, HypreParMatrix _A) -> HyprePCG
        """
        _hypre.HyprePCG_swiginit(self, _hypre.new_HyprePCG(*args))

    def SetOperator(self, op):
        r"""SetOperator(HyprePCG self, Operator op)"""
        return _hypre.HyprePCG_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HyprePCG_SetOperator)

    def SetTol(self, tol):
        r"""SetTol(HyprePCG self, double tol)"""
        return _hypre.HyprePCG_SetTol(self, tol)
    SetTol = _swig_new_instance_method(_hypre.HyprePCG_SetTol)

    def SetMaxIter(self, max_iter):
        r"""SetMaxIter(HyprePCG self, int max_iter)"""
        return _hypre.HyprePCG_SetMaxIter(self, max_iter)
    SetMaxIter = _swig_new_instance_method(_hypre.HyprePCG_SetMaxIter)

    def SetLogging(self, logging):
        r"""SetLogging(HyprePCG self, int logging)"""
        return _hypre.HyprePCG_SetLogging(self, logging)
    SetLogging = _swig_new_instance_method(_hypre.HyprePCG_SetLogging)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(HyprePCG self, int print_lvl)"""
        return _hypre.HyprePCG_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_hypre.HyprePCG_SetPrintLevel)

    def SetPreconditioner(self, precond):
        r"""SetPreconditioner(HyprePCG self, HypreSolver precond)"""
        return _hypre.HyprePCG_SetPreconditioner(self, precond)
    SetPreconditioner = _swig_new_instance_method(_hypre.HyprePCG_SetPreconditioner)

    def SetResidualConvergenceOptions(self, res_frequency=-1, rtol=0.0):
        r"""SetResidualConvergenceOptions(HyprePCG self, int res_frequency=-1, double rtol=0.0)"""
        return _hypre.HyprePCG_SetResidualConvergenceOptions(self, res_frequency, rtol)
    SetResidualConvergenceOptions = _swig_new_instance_method(_hypre.HyprePCG_SetResidualConvergenceOptions)

    def SetZeroInintialIterate(self):
        r"""SetZeroInintialIterate(HyprePCG self)"""
        return _hypre.HyprePCG_SetZeroInintialIterate(self)
    SetZeroInintialIterate = _swig_new_instance_method(_hypre.HyprePCG_SetZeroInintialIterate)

    def SetZeroInitialIterate(self):
        r"""SetZeroInitialIterate(HyprePCG self)"""
        return _hypre.HyprePCG_SetZeroInitialIterate(self)
    SetZeroInitialIterate = _swig_new_instance_method(_hypre.HyprePCG_SetZeroInitialIterate)

    def GetNumIterations(self, num_iterations):
        r"""GetNumIterations(HyprePCG self, int & num_iterations)"""
        return _hypre.HyprePCG_GetNumIterations(self, num_iterations)
    GetNumIterations = _swig_new_instance_method(_hypre.HyprePCG_GetNumIterations)

    def SetupFcn(self):
        r"""SetupFcn(HyprePCG self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HyprePCG_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HyprePCG_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HyprePCG self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HyprePCG_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HyprePCG_SolveFcn)

    def Mult(self, *args):
        r"""
        Mult(HyprePCG self, HypreParVector b, HypreParVector x)
        Mult(HyprePCG self, Vector b, Vector x)
        """
        return _hypre.HyprePCG_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HyprePCG_Mult)
    __swig_destroy__ = _hypre.delete_HyprePCG

# Register HyprePCG in _hypre:
_hypre.HyprePCG_swigregister(HyprePCG)

class HypreGMRES(HypreSolver):
    r"""Proxy of C++ mfem::HypreGMRES class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreGMRES self, MPI_Comm comm) -> HypreGMRES
        __init__(HypreGMRES self, HypreParMatrix _A) -> HypreGMRES
        """
        _hypre.HypreGMRES_swiginit(self, _hypre.new_HypreGMRES(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreGMRES self, Operator op)"""
        return _hypre.HypreGMRES_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreGMRES_SetOperator)

    def SetTol(self, tol):
        r"""SetTol(HypreGMRES self, double tol)"""
        return _hypre.HypreGMRES_SetTol(self, tol)
    SetTol = _swig_new_instance_method(_hypre.HypreGMRES_SetTol)

    def SetMaxIter(self, max_iter):
        r"""SetMaxIter(HypreGMRES self, int max_iter)"""
        return _hypre.HypreGMRES_SetMaxIter(self, max_iter)
    SetMaxIter = _swig_new_instance_method(_hypre.HypreGMRES_SetMaxIter)

    def SetKDim(self, dim):
        r"""SetKDim(HypreGMRES self, int dim)"""
        return _hypre.HypreGMRES_SetKDim(self, dim)
    SetKDim = _swig_new_instance_method(_hypre.HypreGMRES_SetKDim)

    def SetLogging(self, logging):
        r"""SetLogging(HypreGMRES self, int logging)"""
        return _hypre.HypreGMRES_SetLogging(self, logging)
    SetLogging = _swig_new_instance_method(_hypre.HypreGMRES_SetLogging)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(HypreGMRES self, int print_lvl)"""
        return _hypre.HypreGMRES_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreGMRES_SetPrintLevel)

    def SetPreconditioner(self, precond):
        r"""SetPreconditioner(HypreGMRES self, HypreSolver precond)"""
        return _hypre.HypreGMRES_SetPreconditioner(self, precond)
    SetPreconditioner = _swig_new_instance_method(_hypre.HypreGMRES_SetPreconditioner)

    def SetZeroInintialIterate(self):
        r"""SetZeroInintialIterate(HypreGMRES self)"""
        return _hypre.HypreGMRES_SetZeroInintialIterate(self)
    SetZeroInintialIterate = _swig_new_instance_method(_hypre.HypreGMRES_SetZeroInintialIterate)

    def SetZeroInitialIterate(self):
        r"""SetZeroInitialIterate(HypreGMRES self)"""
        return _hypre.HypreGMRES_SetZeroInitialIterate(self)
    SetZeroInitialIterate = _swig_new_instance_method(_hypre.HypreGMRES_SetZeroInitialIterate)

    def SetupFcn(self):
        r"""SetupFcn(HypreGMRES self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreGMRES_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreGMRES_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreGMRES self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreGMRES_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreGMRES_SolveFcn)

    def Mult(self, *args):
        r"""
        Mult(HypreGMRES self, HypreParVector b, HypreParVector x)
        Mult(HypreGMRES self, Vector b, Vector x)
        """
        return _hypre.HypreGMRES_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HypreGMRES_Mult)
    __swig_destroy__ = _hypre.delete_HypreGMRES

# Register HypreGMRES in _hypre:
_hypre.HypreGMRES_swigregister(HypreGMRES)

class HypreFGMRES(HypreSolver):
    r"""Proxy of C++ mfem::HypreFGMRES class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreFGMRES self, MPI_Comm comm) -> HypreFGMRES
        __init__(HypreFGMRES self, HypreParMatrix _A) -> HypreFGMRES
        """
        _hypre.HypreFGMRES_swiginit(self, _hypre.new_HypreFGMRES(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreFGMRES self, Operator op)"""
        return _hypre.HypreFGMRES_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreFGMRES_SetOperator)

    def SetTol(self, tol):
        r"""SetTol(HypreFGMRES self, double tol)"""
        return _hypre.HypreFGMRES_SetTol(self, tol)
    SetTol = _swig_new_instance_method(_hypre.HypreFGMRES_SetTol)

    def SetMaxIter(self, max_iter):
        r"""SetMaxIter(HypreFGMRES self, int max_iter)"""
        return _hypre.HypreFGMRES_SetMaxIter(self, max_iter)
    SetMaxIter = _swig_new_instance_method(_hypre.HypreFGMRES_SetMaxIter)

    def SetKDim(self, dim):
        r"""SetKDim(HypreFGMRES self, int dim)"""
        return _hypre.HypreFGMRES_SetKDim(self, dim)
    SetKDim = _swig_new_instance_method(_hypre.HypreFGMRES_SetKDim)

    def SetLogging(self, logging):
        r"""SetLogging(HypreFGMRES self, int logging)"""
        return _hypre.HypreFGMRES_SetLogging(self, logging)
    SetLogging = _swig_new_instance_method(_hypre.HypreFGMRES_SetLogging)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(HypreFGMRES self, int print_lvl)"""
        return _hypre.HypreFGMRES_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreFGMRES_SetPrintLevel)

    def SetPreconditioner(self, precond):
        r"""SetPreconditioner(HypreFGMRES self, HypreSolver precond)"""
        return _hypre.HypreFGMRES_SetPreconditioner(self, precond)
    SetPreconditioner = _swig_new_instance_method(_hypre.HypreFGMRES_SetPreconditioner)

    def SetZeroInintialIterate(self):
        r"""SetZeroInintialIterate(HypreFGMRES self)"""
        return _hypre.HypreFGMRES_SetZeroInintialIterate(self)
    SetZeroInintialIterate = _swig_new_instance_method(_hypre.HypreFGMRES_SetZeroInintialIterate)

    def SetZeroInitialIterate(self):
        r"""SetZeroInitialIterate(HypreFGMRES self)"""
        return _hypre.HypreFGMRES_SetZeroInitialIterate(self)
    SetZeroInitialIterate = _swig_new_instance_method(_hypre.HypreFGMRES_SetZeroInitialIterate)

    def SetupFcn(self):
        r"""SetupFcn(HypreFGMRES self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreFGMRES_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreFGMRES_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreFGMRES self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreFGMRES_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreFGMRES_SolveFcn)

    def Mult(self, *args):
        r"""
        Mult(HypreFGMRES self, HypreParVector b, HypreParVector x)
        Mult(HypreFGMRES self, Vector b, Vector x)
        """
        return _hypre.HypreFGMRES_Mult(self, *args)
    Mult = _swig_new_instance_method(_hypre.HypreFGMRES_Mult)
    __swig_destroy__ = _hypre.delete_HypreFGMRES

# Register HypreFGMRES in _hypre:
_hypre.HypreFGMRES_swigregister(HypreFGMRES)

class HypreIdentity(HypreSolver):
    r"""Proxy of C++ mfem::HypreIdentity class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def SetupFcn(self):
        r"""SetupFcn(HypreIdentity self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreIdentity_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreIdentity_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreIdentity self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreIdentity_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreIdentity_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreIdentity

    def __init__(self):
        r"""__init__(HypreIdentity self) -> HypreIdentity"""
        _hypre.HypreIdentity_swiginit(self, _hypre.new_HypreIdentity())

# Register HypreIdentity in _hypre:
_hypre.HypreIdentity_swigregister(HypreIdentity)

class HypreDiagScale(HypreSolver):
    r"""Proxy of C++ mfem::HypreDiagScale class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreDiagScale self) -> HypreDiagScale
        __init__(HypreDiagScale self, HypreParMatrix A) -> HypreDiagScale
        """
        _hypre.HypreDiagScale_swiginit(self, _hypre.new_HypreDiagScale(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreDiagScale self, Operator op)"""
        return _hypre.HypreDiagScale_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreDiagScale_SetOperator)

    def SetupFcn(self):
        r"""SetupFcn(HypreDiagScale self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreDiagScale_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreDiagScale_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreDiagScale self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreDiagScale_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreDiagScale_SolveFcn)

    def GetData(self):
        r"""GetData(HypreDiagScale self) -> HypreParMatrix"""
        return _hypre.HypreDiagScale_GetData(self)
    GetData = _swig_new_instance_method(_hypre.HypreDiagScale_GetData)
    __swig_destroy__ = _hypre.delete_HypreDiagScale

# Register HypreDiagScale in _hypre:
_hypre.HypreDiagScale_swigregister(HypreDiagScale)

class HypreParaSails(HypreSolver):
    r"""Proxy of C++ mfem::HypreParaSails class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreParaSails self, MPI_Comm comm) -> HypreParaSails
        __init__(HypreParaSails self, HypreParMatrix A) -> HypreParaSails
        """
        _hypre.HypreParaSails_swiginit(self, _hypre.new_HypreParaSails(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreParaSails self, Operator op)"""
        return _hypre.HypreParaSails_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreParaSails_SetOperator)

    def SetSymmetry(self, sym):
        r"""SetSymmetry(HypreParaSails self, int sym)"""
        return _hypre.HypreParaSails_SetSymmetry(self, sym)
    SetSymmetry = _swig_new_instance_method(_hypre.HypreParaSails_SetSymmetry)

    def SetupFcn(self):
        r"""SetupFcn(HypreParaSails self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreParaSails_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreParaSails_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreParaSails self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreParaSails_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreParaSails_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreParaSails

# Register HypreParaSails in _hypre:
_hypre.HypreParaSails_swigregister(HypreParaSails)

class HypreEuclid(HypreSolver):
    r"""Proxy of C++ mfem::HypreEuclid class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreEuclid self, MPI_Comm comm) -> HypreEuclid
        __init__(HypreEuclid self, HypreParMatrix A) -> HypreEuclid
        """
        _hypre.HypreEuclid_swiginit(self, _hypre.new_HypreEuclid(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreEuclid self, Operator op)"""
        return _hypre.HypreEuclid_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreEuclid_SetOperator)

    def SetupFcn(self):
        r"""SetupFcn(HypreEuclid self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreEuclid_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreEuclid_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreEuclid self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreEuclid_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreEuclid_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreEuclid

# Register HypreEuclid in _hypre:
_hypre.HypreEuclid_swigregister(HypreEuclid)

class HypreBoomerAMG(HypreSolver):
    r"""Proxy of C++ mfem::HypreBoomerAMG class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreBoomerAMG self) -> HypreBoomerAMG
        __init__(HypreBoomerAMG self, HypreParMatrix A) -> HypreBoomerAMG
        """
        _hypre.HypreBoomerAMG_swiginit(self, _hypre.new_HypreBoomerAMG(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreBoomerAMG self, Operator op)"""
        return _hypre.HypreBoomerAMG_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreBoomerAMG_SetOperator)

    def SetSystemsOptions(self, dim, order_bynodes=False):
        r"""SetSystemsOptions(HypreBoomerAMG self, int dim, bool order_bynodes=False)"""
        return _hypre.HypreBoomerAMG_SetSystemsOptions(self, dim, order_bynodes)
    SetSystemsOptions = _swig_new_instance_method(_hypre.HypreBoomerAMG_SetSystemsOptions)

    def SetElasticityOptions(self, fespace):
        r"""SetElasticityOptions(HypreBoomerAMG self, mfem::ParFiniteElementSpace * fespace)"""
        return _hypre.HypreBoomerAMG_SetElasticityOptions(self, fespace)
    SetElasticityOptions = _swig_new_instance_method(_hypre.HypreBoomerAMG_SetElasticityOptions)

    def SetPrintLevel(self, print_level):
        r"""SetPrintLevel(HypreBoomerAMG self, int print_level)"""
        return _hypre.HypreBoomerAMG_SetPrintLevel(self, print_level)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreBoomerAMG_SetPrintLevel)

    def SetupFcn(self):
        r"""SetupFcn(HypreBoomerAMG self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreBoomerAMG_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreBoomerAMG_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreBoomerAMG self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreBoomerAMG_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreBoomerAMG_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreBoomerAMG

# Register HypreBoomerAMG in _hypre:
_hypre.HypreBoomerAMG_swigregister(HypreBoomerAMG)

class HypreAMS(HypreSolver):
    r"""Proxy of C++ mfem::HypreAMS class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreAMS self, mfem::ParFiniteElementSpace * edge_fespace) -> HypreAMS
        __init__(HypreAMS self, HypreParMatrix A, mfem::ParFiniteElementSpace * edge_fespace) -> HypreAMS
        """
        _hypre.HypreAMS_swiginit(self, _hypre.new_HypreAMS(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreAMS self, Operator op)"""
        return _hypre.HypreAMS_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreAMS_SetOperator)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(HypreAMS self, int print_lvl)"""
        return _hypre.HypreAMS_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreAMS_SetPrintLevel)

    def SetSingularProblem(self):
        r"""SetSingularProblem(HypreAMS self)"""
        return _hypre.HypreAMS_SetSingularProblem(self)
    SetSingularProblem = _swig_new_instance_method(_hypre.HypreAMS_SetSingularProblem)

    def SetupFcn(self):
        r"""SetupFcn(HypreAMS self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreAMS_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreAMS_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreAMS self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreAMS_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreAMS_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreAMS

# Register HypreAMS in _hypre:
_hypre.HypreAMS_swigregister(HypreAMS)

class HypreADS(HypreSolver):
    r"""Proxy of C++ mfem::HypreADS class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(HypreADS self, mfem::ParFiniteElementSpace * face_fespace) -> HypreADS
        __init__(HypreADS self, HypreParMatrix A, mfem::ParFiniteElementSpace * face_fespace) -> HypreADS
        """
        _hypre.HypreADS_swiginit(self, _hypre.new_HypreADS(*args))

    def SetOperator(self, op):
        r"""SetOperator(HypreADS self, Operator op)"""
        return _hypre.HypreADS_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_hypre.HypreADS_SetOperator)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(HypreADS self, int print_lvl)"""
        return _hypre.HypreADS_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreADS_SetPrintLevel)

    def SetupFcn(self):
        r"""SetupFcn(HypreADS self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreADS_SetupFcn(self)
    SetupFcn = _swig_new_instance_method(_hypre.HypreADS_SetupFcn)

    def SolveFcn(self):
        r"""SolveFcn(HypreADS self) -> HYPRE_PtrToParSolverFcn"""
        return _hypre.HypreADS_SolveFcn(self)
    SolveFcn = _swig_new_instance_method(_hypre.HypreADS_SolveFcn)
    __swig_destroy__ = _hypre.delete_HypreADS

# Register HypreADS in _hypre:
_hypre.HypreADS_swigregister(HypreADS)

class HypreLOBPCG(object):
    r"""Proxy of C++ mfem::HypreLOBPCG class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, comm):
        r"""__init__(HypreLOBPCG self, MPI_Comm comm) -> HypreLOBPCG"""
        _hypre.HypreLOBPCG_swiginit(self, _hypre.new_HypreLOBPCG(comm))
    __swig_destroy__ = _hypre.delete_HypreLOBPCG

    def SetTol(self, tol):
        r"""SetTol(HypreLOBPCG self, double tol)"""
        return _hypre.HypreLOBPCG_SetTol(self, tol)
    SetTol = _swig_new_instance_method(_hypre.HypreLOBPCG_SetTol)

    def SetRelTol(self, rel_tol):
        r"""SetRelTol(HypreLOBPCG self, double rel_tol)"""
        return _hypre.HypreLOBPCG_SetRelTol(self, rel_tol)
    SetRelTol = _swig_new_instance_method(_hypre.HypreLOBPCG_SetRelTol)

    def SetMaxIter(self, max_iter):
        r"""SetMaxIter(HypreLOBPCG self, int max_iter)"""
        return _hypre.HypreLOBPCG_SetMaxIter(self, max_iter)
    SetMaxIter = _swig_new_instance_method(_hypre.HypreLOBPCG_SetMaxIter)

    def SetPrintLevel(self, logging):
        r"""SetPrintLevel(HypreLOBPCG self, int logging)"""
        return _hypre.HypreLOBPCG_SetPrintLevel(self, logging)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreLOBPCG_SetPrintLevel)

    def SetNumModes(self, num_eigs):
        r"""SetNumModes(HypreLOBPCG self, int num_eigs)"""
        return _hypre.HypreLOBPCG_SetNumModes(self, num_eigs)
    SetNumModes = _swig_new_instance_method(_hypre.HypreLOBPCG_SetNumModes)

    def SetPrecondUsageMode(self, pcg_mode):
        r"""SetPrecondUsageMode(HypreLOBPCG self, int pcg_mode)"""
        return _hypre.HypreLOBPCG_SetPrecondUsageMode(self, pcg_mode)
    SetPrecondUsageMode = _swig_new_instance_method(_hypre.HypreLOBPCG_SetPrecondUsageMode)

    def SetRandomSeed(self, s):
        r"""SetRandomSeed(HypreLOBPCG self, int s)"""
        return _hypre.HypreLOBPCG_SetRandomSeed(self, s)
    SetRandomSeed = _swig_new_instance_method(_hypre.HypreLOBPCG_SetRandomSeed)

    def SetInitialVectors(self, num_vecs, vecs):
        r"""SetInitialVectors(HypreLOBPCG self, int num_vecs, mfem::HypreParVector ** vecs)"""
        return _hypre.HypreLOBPCG_SetInitialVectors(self, num_vecs, vecs)
    SetInitialVectors = _swig_new_instance_method(_hypre.HypreLOBPCG_SetInitialVectors)

    def SetPreconditioner(self, precond):
        r"""SetPreconditioner(HypreLOBPCG self, Solver precond)"""
        return _hypre.HypreLOBPCG_SetPreconditioner(self, precond)
    SetPreconditioner = _swig_new_instance_method(_hypre.HypreLOBPCG_SetPreconditioner)

    def SetOperator(self, A):
        r"""SetOperator(HypreLOBPCG self, Operator A)"""
        return _hypre.HypreLOBPCG_SetOperator(self, A)
    SetOperator = _swig_new_instance_method(_hypre.HypreLOBPCG_SetOperator)

    def SetMassMatrix(self, M):
        r"""SetMassMatrix(HypreLOBPCG self, Operator M)"""
        return _hypre.HypreLOBPCG_SetMassMatrix(self, M)
    SetMassMatrix = _swig_new_instance_method(_hypre.HypreLOBPCG_SetMassMatrix)

    def SetSubSpaceProjector(self, proj):
        r"""SetSubSpaceProjector(HypreLOBPCG self, Operator proj)"""
        return _hypre.HypreLOBPCG_SetSubSpaceProjector(self, proj)
    SetSubSpaceProjector = _swig_new_instance_method(_hypre.HypreLOBPCG_SetSubSpaceProjector)

    def Solve(self):
        r"""Solve(HypreLOBPCG self)"""
        return _hypre.HypreLOBPCG_Solve(self)
    Solve = _swig_new_instance_method(_hypre.HypreLOBPCG_Solve)

    def GetEigenvalues(self, eigenvalues):
        r"""GetEigenvalues(HypreLOBPCG self, doubleArray eigenvalues)"""
        return _hypre.HypreLOBPCG_GetEigenvalues(self, eigenvalues)
    GetEigenvalues = _swig_new_instance_method(_hypre.HypreLOBPCG_GetEigenvalues)

    def GetEigenvector(self, i):
        r"""GetEigenvector(HypreLOBPCG self, unsigned int i) -> HypreParVector"""
        return _hypre.HypreLOBPCG_GetEigenvector(self, i)
    GetEigenvector = _swig_new_instance_method(_hypre.HypreLOBPCG_GetEigenvector)

    def StealEigenvectors(self):
        r"""StealEigenvectors(HypreLOBPCG self) -> mfem::HypreParVector **"""
        return _hypre.HypreLOBPCG_StealEigenvectors(self)
    StealEigenvectors = _swig_new_instance_method(_hypre.HypreLOBPCG_StealEigenvectors)

# Register HypreLOBPCG in _hypre:
_hypre.HypreLOBPCG_swigregister(HypreLOBPCG)

class HypreAME(object):
    r"""Proxy of C++ mfem::HypreAME class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, comm):
        r"""__init__(HypreAME self, MPI_Comm comm) -> HypreAME"""
        _hypre.HypreAME_swiginit(self, _hypre.new_HypreAME(comm))
    __swig_destroy__ = _hypre.delete_HypreAME

    def SetTol(self, tol):
        r"""SetTol(HypreAME self, double tol)"""
        return _hypre.HypreAME_SetTol(self, tol)
    SetTol = _swig_new_instance_method(_hypre.HypreAME_SetTol)

    def SetRelTol(self, rel_tol):
        r"""SetRelTol(HypreAME self, double rel_tol)"""
        return _hypre.HypreAME_SetRelTol(self, rel_tol)
    SetRelTol = _swig_new_instance_method(_hypre.HypreAME_SetRelTol)

    def SetMaxIter(self, max_iter):
        r"""SetMaxIter(HypreAME self, int max_iter)"""
        return _hypre.HypreAME_SetMaxIter(self, max_iter)
    SetMaxIter = _swig_new_instance_method(_hypre.HypreAME_SetMaxIter)

    def SetPrintLevel(self, logging):
        r"""SetPrintLevel(HypreAME self, int logging)"""
        return _hypre.HypreAME_SetPrintLevel(self, logging)
    SetPrintLevel = _swig_new_instance_method(_hypre.HypreAME_SetPrintLevel)

    def SetNumModes(self, num_eigs):
        r"""SetNumModes(HypreAME self, int num_eigs)"""
        return _hypre.HypreAME_SetNumModes(self, num_eigs)
    SetNumModes = _swig_new_instance_method(_hypre.HypreAME_SetNumModes)

    def SetPreconditioner(self, precond):
        r"""SetPreconditioner(HypreAME self, HypreSolver precond)"""
        return _hypre.HypreAME_SetPreconditioner(self, precond)
    SetPreconditioner = _swig_new_instance_method(_hypre.HypreAME_SetPreconditioner)

    def SetOperator(self, A):
        r"""SetOperator(HypreAME self, HypreParMatrix A)"""
        return _hypre.HypreAME_SetOperator(self, A)
    SetOperator = _swig_new_instance_method(_hypre.HypreAME_SetOperator)

    def SetMassMatrix(self, M):
        r"""SetMassMatrix(HypreAME self, HypreParMatrix M)"""
        return _hypre.HypreAME_SetMassMatrix(self, M)
    SetMassMatrix = _swig_new_instance_method(_hypre.HypreAME_SetMassMatrix)

    def Solve(self):
        r"""Solve(HypreAME self)"""
        return _hypre.HypreAME_Solve(self)
    Solve = _swig_new_instance_method(_hypre.HypreAME_Solve)

    def GetEigenvalues(self, eigenvalues):
        r"""GetEigenvalues(HypreAME self, doubleArray eigenvalues)"""
        return _hypre.HypreAME_GetEigenvalues(self, eigenvalues)
    GetEigenvalues = _swig_new_instance_method(_hypre.HypreAME_GetEigenvalues)

    def GetEigenvector(self, i):
        r"""GetEigenvector(HypreAME self, unsigned int i) -> HypreParVector"""
        return _hypre.HypreAME_GetEigenvector(self, i)
    GetEigenvector = _swig_new_instance_method(_hypre.HypreAME_GetEigenvector)

    def StealEigenvectors(self):
        r"""StealEigenvectors(HypreAME self) -> mfem::HypreParVector **"""
        return _hypre.HypreAME_StealEigenvectors(self)
    StealEigenvectors = _swig_new_instance_method(_hypre.HypreAME_StealEigenvectors)

# Register HypreAME in _hypre:
_hypre.HypreAME_swigregister(HypreAME)


def parvec__repr__(self):
    return "HypreParVector ("+str(self.GlobalSize())+")"
def parvec__del__(self):
    if hasattr(self, "_linked_array"):
        self._linked_array = None
def parmat__repr__(self):
    shape = (self.GetGlobalNumRows(), self.GetGlobalNumCols())
    lshape = (self.GetNumRows(), self.GetNumCols())  	       
    return "HypreParMatrix "+str(shape)+"["+str(lshape)+"]"


HypreParVector.__repr__ = parvec__repr__
HypreParVector.__del__  = parvec__del__      
HypreParMatrix.__repr__ = parmat__repr__






'''
   MFEM example 13p

    This example code solves the Maxwell (electromagnetic)
    eigenvalue problem curl curl E = lambda E with homogeneous
    Dirichlet boundary conditions E x n = 0.

   See c++ version in the MFEM library for more detail 
'''
from mfem import path
import mfem.par as mfem
from mfem.par import intArray, doubleArray
from os.path import expanduser, join
from mpi4py import MPI
import numpy as np
from numpy import sin, cos, exp

ser_ref_levels = 2
par_ref_levels = 1
order = 1
nev = 5
visualization = 1

num_proc = MPI.COMM_WORLD.size
myid     = MPI.COMM_WORLD.rank

meshfile = expanduser(join(path, 'data', 'beam-tet.mesh'))
mesh = mfem.Mesh(meshfile, 1,1)

dim = mesh.Dimension()

for k in range(ser_ref_levels):
    mesh.UniformRefinement()

pmesh = mfem.ParMesh(MPI.COMM_WORLD, mesh)
for k in range(par_ref_levels):
    pmesh.UniformRefinement()
pmesh.ReorientTetMesh()
fec = mfem.ND_FECollection(order, dim)
fespace = mfem.ParFiniteElementSpace(pmesh, fec)
size = fespace.GlobalTrueVSize()
if (myid == 0):
    print("Number of unknowns: " + str(size))

one = mfem.ConstantCoefficient(1.0);
ess_bdr = intArray()
if (pmesh.bdr_attributes.Size() > 0):
   ess_bdr.SetSize(pmesh.bdr_attributes.Max())
   ess_bdr.Assign(1)

a = mfem.ParBilinearForm(fespace)
a.AddDomainIntegrator(mfem.CurlCurlIntegrator(one))

if (pmesh.bdr_attributes.Size() == 0):
     # Add a mass term if the mesh has no boundary, e.g. periodic mesh or
     # closed surface.
     a.AddDomainIntegrator(mfem.VectorFEMassIntegrator(one))

a.Assemble();
a.EliminateEssentialBCDiag(ess_bdr, 1.0);
a.Finalize();

m = mfem.ParBilinearForm(fespace)
m.AddDomainIntegrator(mfem.VectorFEMassIntegrator(one))
m.Assemble()
# shift the eigenvalue corresponding to eliminated dofs to a large value
m.EliminateEssentialBCDiag(ess_bdr, 2.3e-308)
m.Finalize();

A = a.ParallelAssemble()
M = m.ParallelAssemble()
   
ams = mfem.HypreAMS(A,fespace)
ams.SetPrintLevel(0);
ams.SetSingularProblem();

ame = mfem.HypreAME(MPI.COMM_WORLD)
ame.SetNumModes(nev)
ame.SetPreconditioner(ams)
ame.SetMaxIter(100)
ame.SetTol(1e-8)
ame.SetPrintLevel(1)
ame.SetMassMatrix(M)
ame.SetOperator(A)

eigenvalues = doubleArray()
ame.Solve();
ame.GetEigenvalues(eigenvalues);
x = mfem.ParGridFunction(fespace)


smyid = '{:0>6d}'.format(myid)
mesh_name  =  "ex13_mesh."+smyid


pmesh.Print(mesh_name, 8)

for i in range(nev):
    if ( myid == 0 ):
        print("Eigenmode " + str(i+1)  +'/' + str(nev) +
              ", Lambda = " + str(eigenvalues[i]))
    
    sol_name   =  "ex13_mode_"+str(i)+"."+smyid    
    x.Assign(ame.GetEigenvector(i))
    x.Save(sol_name, 8)    
    c  = None
    if (myid == 0):
        from builtins import input
        c = input("press (q)uit or (c)ontinue --> ")
    c = MPI.COMM_WORLD.bcast(c, root=0)
    if (c != 'c'): break





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _mem_manager
else:
    import _mem_manager

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _mem_manager.SWIG_PyInstanceMethod_New
_swig_new_static_method = _mem_manager.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


MemoryType_HOST = _mem_manager.MemoryType_HOST
MemoryType_HOST_32 = _mem_manager.MemoryType_HOST_32
MemoryType_HOST_64 = _mem_manager.MemoryType_HOST_64
MemoryType_HOST_DEBUG = _mem_manager.MemoryType_HOST_DEBUG
MemoryType_HOST_UMPIRE = _mem_manager.MemoryType_HOST_UMPIRE
MemoryType_MANAGED = _mem_manager.MemoryType_MANAGED
MemoryType_DEVICE = _mem_manager.MemoryType_DEVICE
MemoryType_DEVICE_DEBUG = _mem_manager.MemoryType_DEVICE_DEBUG
MemoryType_DEVICE_UMPIRE = _mem_manager.MemoryType_DEVICE_UMPIRE
MemoryType_SIZE = _mem_manager.MemoryType_SIZE
MemoryClass_HOST = _mem_manager.MemoryClass_HOST
MemoryClass_HOST_32 = _mem_manager.MemoryClass_HOST_32
MemoryClass_HOST_64 = _mem_manager.MemoryClass_HOST_64
MemoryClass_DEVICE = _mem_manager.MemoryClass_DEVICE
MemoryClass_MANAGED = _mem_manager.MemoryClass_MANAGED

def IsHostMemory(mt):
    return _mem_manager.IsHostMemory(mt)
IsHostMemory = _mem_manager.IsHostMemory

def IsDeviceMemory(mt):
    return _mem_manager.IsDeviceMemory(mt)
IsDeviceMemory = _mem_manager.IsDeviceMemory

def GetMemoryType(mc):
    return _mem_manager.GetMemoryType(mc)
GetMemoryType = _mem_manager.GetMemoryType

def __mul__(mc1, mc2):
    return _mem_manager.__mul__(mc1, mc2)
__mul__ = _mem_manager.__mul__
class MemoryManager(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _mem_manager.MemoryManager_swiginit(self, _mem_manager.new_MemoryManager())
    __swig_destroy__ = _mem_manager.delete_MemoryManager

    def Init(self):
        return _mem_manager.MemoryManager_Init(self)
    Init = _swig_new_instance_method(_mem_manager.MemoryManager_Init)

    def Configure(self, h_mt, d_mt):
        return _mem_manager.MemoryManager_Configure(self, h_mt, d_mt)
    Configure = _swig_new_instance_method(_mem_manager.MemoryManager_Configure)

    def Destroy(self):
        return _mem_manager.MemoryManager_Destroy(self)
    Destroy = _swig_new_instance_method(_mem_manager.MemoryManager_Destroy)

    def IsKnown(self, h_ptr):
        return _mem_manager.MemoryManager_IsKnown(self, h_ptr)
    IsKnown = _swig_new_instance_method(_mem_manager.MemoryManager_IsKnown)

    def IsAlias(self, h_ptr):
        return _mem_manager.MemoryManager_IsAlias(self, h_ptr)
    IsAlias = _swig_new_instance_method(_mem_manager.MemoryManager_IsAlias)

    def RegisterCheck(self, h_ptr):
        return _mem_manager.MemoryManager_RegisterCheck(self, h_ptr)
    RegisterCheck = _swig_new_instance_method(_mem_manager.MemoryManager_RegisterCheck)

    def PrintPtrs(self, *args, **kwargs):
        return _mem_manager.MemoryManager_PrintPtrs(self, *args, **kwargs)
    PrintPtrs = _swig_new_instance_method(_mem_manager.MemoryManager_PrintPtrs)

    def PrintAliases(self, *args, **kwargs):
        return _mem_manager.MemoryManager_PrintAliases(self, *args, **kwargs)
    PrintAliases = _swig_new_instance_method(_mem_manager.MemoryManager_PrintAliases)

    @staticmethod
    def GetHostMemoryType():
        return _mem_manager.MemoryManager_GetHostMemoryType()
    GetHostMemoryType = _swig_new_static_method(_mem_manager.MemoryManager_GetHostMemoryType)

    @staticmethod
    def GetDeviceMemoryType():
        return _mem_manager.MemoryManager_GetDeviceMemoryType()
    GetDeviceMemoryType = _swig_new_static_method(_mem_manager.MemoryManager_GetDeviceMemoryType)

# Register MemoryManager in _mem_manager:
_mem_manager.MemoryManager_swigregister(MemoryManager)
cvar = _mem_manager.cvar
MemoryTypeSize = cvar.MemoryTypeSize
HostMemoryType = cvar.HostMemoryType
HostMemoryTypeSize = cvar.HostMemoryTypeSize
DeviceMemoryType = cvar.DeviceMemoryType
DeviceMemoryTypeSize = cvar.DeviceMemoryTypeSize

def MemoryManager_GetHostMemoryType():
    return _mem_manager.MemoryManager_GetHostMemoryType()
MemoryManager_GetHostMemoryType = _mem_manager.MemoryManager_GetHostMemoryType

def MemoryManager_GetDeviceMemoryType():
    return _mem_manager.MemoryManager_GetDeviceMemoryType()
MemoryManager_GetDeviceMemoryType = _mem_manager.MemoryManager_GetDeviceMemoryType


def MemoryPrintFlags(flags):
    return _mem_manager.MemoryPrintFlags(flags)
MemoryPrintFlags = _mem_manager.MemoryPrintFlags





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _nonlininteg
else:
    import _nonlininteg

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _nonlininteg.SWIG_PyInstanceMethod_New
_swig_new_static_method = _nonlininteg.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
class NonlinearFormIntegrator(object):
    r"""Proxy of C++ mfem::NonlinearFormIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, ir=None):
        r"""__init__(NonlinearFormIntegrator self, IntegrationRule ir=None) -> NonlinearFormIntegrator"""
        if self.__class__ == NonlinearFormIntegrator:
            _self = None
        else:
            _self = self
        _nonlininteg.NonlinearFormIntegrator_swiginit(self, _nonlininteg.new_NonlinearFormIntegrator(_self, ir))

    def SetIntRule(self, ir):
        r"""SetIntRule(NonlinearFormIntegrator self, IntegrationRule ir)"""
        return _nonlininteg.NonlinearFormIntegrator_SetIntRule(self, ir)
    SetIntRule = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_SetIntRule)

    def SetIntegrationRule(self, irule):
        r"""SetIntegrationRule(NonlinearFormIntegrator self, IntegrationRule irule)"""
        return _nonlininteg.NonlinearFormIntegrator_SetIntegrationRule(self, irule)
    SetIntegrationRule = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_SetIntegrationRule)

    def AssembleElementVector(self, el, Tr, elfun, elvect):
        r"""AssembleElementVector(NonlinearFormIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elfun, Vector elvect)"""
        return _nonlininteg.NonlinearFormIntegrator_AssembleElementVector(self, el, Tr, elfun, elvect)
    AssembleElementVector = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AssembleElementVector)

    def AssembleFaceVector(self, el1, el2, Tr, elfun, elvect):
        r"""AssembleFaceVector(NonlinearFormIntegrator self, FiniteElement el1, FiniteElement el2, FaceElementTransformations Tr, Vector elfun, Vector elvect)"""
        return _nonlininteg.NonlinearFormIntegrator_AssembleFaceVector(self, el1, el2, Tr, elfun, elvect)
    AssembleFaceVector = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AssembleFaceVector)

    def AssembleElementGrad(self, el, Tr, elfun, elmat):
        r"""AssembleElementGrad(NonlinearFormIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elfun, DenseMatrix elmat)"""
        return _nonlininteg.NonlinearFormIntegrator_AssembleElementGrad(self, el, Tr, elfun, elmat)
    AssembleElementGrad = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AssembleElementGrad)

    def AssembleFaceGrad(self, el1, el2, Tr, elfun, elmat):
        r"""AssembleFaceGrad(NonlinearFormIntegrator self, FiniteElement el1, FiniteElement el2, FaceElementTransformations Tr, Vector elfun, DenseMatrix elmat)"""
        return _nonlininteg.NonlinearFormIntegrator_AssembleFaceGrad(self, el1, el2, Tr, elfun, elmat)
    AssembleFaceGrad = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AssembleFaceGrad)

    def GetElementEnergy(self, el, Tr, elfun):
        r"""GetElementEnergy(NonlinearFormIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elfun) -> double"""
        return _nonlininteg.NonlinearFormIntegrator_GetElementEnergy(self, el, Tr, elfun)
    GetElementEnergy = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_GetElementEnergy)

    def AssemblePA(self, *args):
        r"""
        AssemblePA(NonlinearFormIntegrator self, FiniteElementSpace fes)
        AssemblePA(NonlinearFormIntegrator self, FiniteElementSpace trial_fes, FiniteElementSpace test_fes)
        """
        return _nonlininteg.NonlinearFormIntegrator_AssemblePA(self, *args)
    AssemblePA = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AssemblePA)

    def AddMultPA(self, x, y):
        r"""AddMultPA(NonlinearFormIntegrator self, Vector x, Vector y)"""
        return _nonlininteg.NonlinearFormIntegrator_AddMultPA(self, x, y)
    AddMultPA = _swig_new_instance_method(_nonlininteg.NonlinearFormIntegrator_AddMultPA)
    __swig_destroy__ = _nonlininteg.delete_NonlinearFormIntegrator
    def __disown__(self):
        self.this.disown()
        _nonlininteg.disown_NonlinearFormIntegrator(self)
        return weakref.proxy(self)

# Register NonlinearFormIntegrator in _nonlininteg:
_nonlininteg.NonlinearFormIntegrator_swigregister(NonlinearFormIntegrator)

class BlockNonlinearFormIntegrator(object):
    r"""Proxy of C++ mfem::BlockNonlinearFormIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def GetElementEnergy(self, el, Tr, elfun):
        r"""GetElementEnergy(BlockNonlinearFormIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun) -> double"""
        return _nonlininteg.BlockNonlinearFormIntegrator_GetElementEnergy(self, el, Tr, elfun)
    GetElementEnergy = _swig_new_instance_method(_nonlininteg.BlockNonlinearFormIntegrator_GetElementEnergy)

    def AssembleElementVector(self, el, Tr, elfun, elvec):
        r"""AssembleElementVector(BlockNonlinearFormIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array< mfem::Vector * > const & elvec)"""
        return _nonlininteg.BlockNonlinearFormIntegrator_AssembleElementVector(self, el, Tr, elfun, elvec)
    AssembleElementVector = _swig_new_instance_method(_nonlininteg.BlockNonlinearFormIntegrator_AssembleElementVector)

    def AssembleFaceVector(self, el1, el2, Tr, elfun, elvect):
        r"""AssembleFaceVector(BlockNonlinearFormIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el1, mfem::Array< mfem::FiniteElement const * > const & el2, FaceElementTransformations Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array< mfem::Vector * > const & elvect)"""
        return _nonlininteg.BlockNonlinearFormIntegrator_AssembleFaceVector(self, el1, el2, Tr, elfun, elvect)
    AssembleFaceVector = _swig_new_instance_method(_nonlininteg.BlockNonlinearFormIntegrator_AssembleFaceVector)

    def AssembleElementGrad(self, el, Tr, elfun, elmats):
        r"""AssembleElementGrad(BlockNonlinearFormIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array2D< mfem::DenseMatrix * > const & elmats)"""
        return _nonlininteg.BlockNonlinearFormIntegrator_AssembleElementGrad(self, el, Tr, elfun, elmats)
    AssembleElementGrad = _swig_new_instance_method(_nonlininteg.BlockNonlinearFormIntegrator_AssembleElementGrad)

    def AssembleFaceGrad(self, el1, el2, Tr, elfun, elmats):
        r"""AssembleFaceGrad(BlockNonlinearFormIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el1, mfem::Array< mfem::FiniteElement const * > const & el2, FaceElementTransformations Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array2D< mfem::DenseMatrix * > const & elmats)"""
        return _nonlininteg.BlockNonlinearFormIntegrator_AssembleFaceGrad(self, el1, el2, Tr, elfun, elmats)
    AssembleFaceGrad = _swig_new_instance_method(_nonlininteg.BlockNonlinearFormIntegrator_AssembleFaceGrad)
    __swig_destroy__ = _nonlininteg.delete_BlockNonlinearFormIntegrator

    def __init__(self):
        r"""__init__(BlockNonlinearFormIntegrator self) -> BlockNonlinearFormIntegrator"""
        _nonlininteg.BlockNonlinearFormIntegrator_swiginit(self, _nonlininteg.new_BlockNonlinearFormIntegrator())

# Register BlockNonlinearFormIntegrator in _nonlininteg:
_nonlininteg.BlockNonlinearFormIntegrator_swigregister(BlockNonlinearFormIntegrator)

class HyperelasticModel(object):
    r"""Proxy of C++ mfem::HyperelasticModel class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _nonlininteg.delete_HyperelasticModel

    def SetTransformation(self, _Ttr):
        r"""SetTransformation(HyperelasticModel self, ElementTransformation _Ttr)"""
        return _nonlininteg.HyperelasticModel_SetTransformation(self, _Ttr)
    SetTransformation = _swig_new_instance_method(_nonlininteg.HyperelasticModel_SetTransformation)

    def EvalW(self, Jpt):
        r"""EvalW(HyperelasticModel self, DenseMatrix Jpt) -> double"""
        return _nonlininteg.HyperelasticModel_EvalW(self, Jpt)
    EvalW = _swig_new_instance_method(_nonlininteg.HyperelasticModel_EvalW)

    def EvalP(self, Jpt, P):
        r"""EvalP(HyperelasticModel self, DenseMatrix Jpt, DenseMatrix P)"""
        return _nonlininteg.HyperelasticModel_EvalP(self, Jpt, P)
    EvalP = _swig_new_instance_method(_nonlininteg.HyperelasticModel_EvalP)

    def AssembleH(self, Jpt, DS, weight, A):
        r"""AssembleH(HyperelasticModel self, DenseMatrix Jpt, DenseMatrix DS, double const weight, DenseMatrix A)"""
        return _nonlininteg.HyperelasticModel_AssembleH(self, Jpt, DS, weight, A)
    AssembleH = _swig_new_instance_method(_nonlininteg.HyperelasticModel_AssembleH)

# Register HyperelasticModel in _nonlininteg:
_nonlininteg.HyperelasticModel_swigregister(HyperelasticModel)

class InverseHarmonicModel(HyperelasticModel):
    r"""Proxy of C++ mfem::InverseHarmonicModel class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def EvalW(self, J):
        r"""EvalW(InverseHarmonicModel self, DenseMatrix J) -> double"""
        return _nonlininteg.InverseHarmonicModel_EvalW(self, J)
    EvalW = _swig_new_instance_method(_nonlininteg.InverseHarmonicModel_EvalW)

    def EvalP(self, J, P):
        r"""EvalP(InverseHarmonicModel self, DenseMatrix J, DenseMatrix P)"""
        return _nonlininteg.InverseHarmonicModel_EvalP(self, J, P)
    EvalP = _swig_new_instance_method(_nonlininteg.InverseHarmonicModel_EvalP)

    def AssembleH(self, J, DS, weight, A):
        r"""AssembleH(InverseHarmonicModel self, DenseMatrix J, DenseMatrix DS, double const weight, DenseMatrix A)"""
        return _nonlininteg.InverseHarmonicModel_AssembleH(self, J, DS, weight, A)
    AssembleH = _swig_new_instance_method(_nonlininteg.InverseHarmonicModel_AssembleH)

    def __init__(self):
        r"""__init__(InverseHarmonicModel self) -> InverseHarmonicModel"""
        _nonlininteg.InverseHarmonicModel_swiginit(self, _nonlininteg.new_InverseHarmonicModel())
    __swig_destroy__ = _nonlininteg.delete_InverseHarmonicModel

# Register InverseHarmonicModel in _nonlininteg:
_nonlininteg.InverseHarmonicModel_swigregister(InverseHarmonicModel)

class NeoHookeanModel(HyperelasticModel):
    r"""Proxy of C++ mfem::NeoHookeanModel class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(NeoHookeanModel self, double _mu, double _K, double _g=1.0) -> NeoHookeanModel
        __init__(NeoHookeanModel self, Coefficient _mu, Coefficient _K, Coefficient _g=None) -> NeoHookeanModel
        """
        _nonlininteg.NeoHookeanModel_swiginit(self, _nonlininteg.new_NeoHookeanModel(*args))

    def EvalW(self, J):
        r"""EvalW(NeoHookeanModel self, DenseMatrix J) -> double"""
        return _nonlininteg.NeoHookeanModel_EvalW(self, J)
    EvalW = _swig_new_instance_method(_nonlininteg.NeoHookeanModel_EvalW)

    def EvalP(self, J, P):
        r"""EvalP(NeoHookeanModel self, DenseMatrix J, DenseMatrix P)"""
        return _nonlininteg.NeoHookeanModel_EvalP(self, J, P)
    EvalP = _swig_new_instance_method(_nonlininteg.NeoHookeanModel_EvalP)

    def AssembleH(self, J, DS, weight, A):
        r"""AssembleH(NeoHookeanModel self, DenseMatrix J, DenseMatrix DS, double const weight, DenseMatrix A)"""
        return _nonlininteg.NeoHookeanModel_AssembleH(self, J, DS, weight, A)
    AssembleH = _swig_new_instance_method(_nonlininteg.NeoHookeanModel_AssembleH)
    __swig_destroy__ = _nonlininteg.delete_NeoHookeanModel

# Register NeoHookeanModel in _nonlininteg:
_nonlininteg.NeoHookeanModel_swigregister(NeoHookeanModel)

class HyperelasticNLFIntegrator(NonlinearFormIntegrator):
    r"""Proxy of C++ mfem::HyperelasticNLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, m):
        r"""__init__(HyperelasticNLFIntegrator self, HyperelasticModel m) -> HyperelasticNLFIntegrator"""
        _nonlininteg.HyperelasticNLFIntegrator_swiginit(self, _nonlininteg.new_HyperelasticNLFIntegrator(m))

    def GetElementEnergy(self, el, Ttr, elfun):
        r"""GetElementEnergy(HyperelasticNLFIntegrator self, FiniteElement el, ElementTransformation Ttr, Vector elfun) -> double"""
        return _nonlininteg.HyperelasticNLFIntegrator_GetElementEnergy(self, el, Ttr, elfun)
    GetElementEnergy = _swig_new_instance_method(_nonlininteg.HyperelasticNLFIntegrator_GetElementEnergy)

    def AssembleElementVector(self, el, Ttr, elfun, elvect):
        r"""AssembleElementVector(HyperelasticNLFIntegrator self, FiniteElement el, ElementTransformation Ttr, Vector elfun, Vector elvect)"""
        return _nonlininteg.HyperelasticNLFIntegrator_AssembleElementVector(self, el, Ttr, elfun, elvect)
    AssembleElementVector = _swig_new_instance_method(_nonlininteg.HyperelasticNLFIntegrator_AssembleElementVector)

    def AssembleElementGrad(self, el, Ttr, elfun, elmat):
        r"""AssembleElementGrad(HyperelasticNLFIntegrator self, FiniteElement el, ElementTransformation Ttr, Vector elfun, DenseMatrix elmat)"""
        return _nonlininteg.HyperelasticNLFIntegrator_AssembleElementGrad(self, el, Ttr, elfun, elmat)
    AssembleElementGrad = _swig_new_instance_method(_nonlininteg.HyperelasticNLFIntegrator_AssembleElementGrad)
    __swig_destroy__ = _nonlininteg.delete_HyperelasticNLFIntegrator

# Register HyperelasticNLFIntegrator in _nonlininteg:
_nonlininteg.HyperelasticNLFIntegrator_swigregister(HyperelasticNLFIntegrator)

class IncompressibleNeoHookeanIntegrator(BlockNonlinearFormIntegrator):
    r"""Proxy of C++ mfem::IncompressibleNeoHookeanIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, _mu):
        r"""__init__(IncompressibleNeoHookeanIntegrator self, Coefficient _mu) -> IncompressibleNeoHookeanIntegrator"""
        _nonlininteg.IncompressibleNeoHookeanIntegrator_swiginit(self, _nonlininteg.new_IncompressibleNeoHookeanIntegrator(_mu))

    def GetElementEnergy(self, el, Tr, elfun):
        r"""GetElementEnergy(IncompressibleNeoHookeanIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun) -> double"""
        return _nonlininteg.IncompressibleNeoHookeanIntegrator_GetElementEnergy(self, el, Tr, elfun)
    GetElementEnergy = _swig_new_instance_method(_nonlininteg.IncompressibleNeoHookeanIntegrator_GetElementEnergy)

    def AssembleElementVector(self, el, Tr, elfun, elvec):
        r"""AssembleElementVector(IncompressibleNeoHookeanIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array< mfem::Vector * > const & elvec)"""
        return _nonlininteg.IncompressibleNeoHookeanIntegrator_AssembleElementVector(self, el, Tr, elfun, elvec)
    AssembleElementVector = _swig_new_instance_method(_nonlininteg.IncompressibleNeoHookeanIntegrator_AssembleElementVector)

    def AssembleElementGrad(self, el, Tr, elfun, elmats):
        r"""AssembleElementGrad(IncompressibleNeoHookeanIntegrator self, mfem::Array< mfem::FiniteElement const * > const & el, ElementTransformation Tr, mfem::Array< mfem::Vector const * > const & elfun, mfem::Array2D< mfem::DenseMatrix * > const & elmats)"""
        return _nonlininteg.IncompressibleNeoHookeanIntegrator_AssembleElementGrad(self, el, Tr, elfun, elmats)
    AssembleElementGrad = _swig_new_instance_method(_nonlininteg.IncompressibleNeoHookeanIntegrator_AssembleElementGrad)
    __swig_destroy__ = _nonlininteg.delete_IncompressibleNeoHookeanIntegrator

# Register IncompressibleNeoHookeanIntegrator in _nonlininteg:
_nonlininteg.IncompressibleNeoHookeanIntegrator_swigregister(IncompressibleNeoHookeanIntegrator)

class VectorConvectionNLFIntegrator(NonlinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorConvectionNLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(VectorConvectionNLFIntegrator self, Coefficient q) -> VectorConvectionNLFIntegrator
        __init__(VectorConvectionNLFIntegrator self) -> VectorConvectionNLFIntegrator
        """
        _nonlininteg.VectorConvectionNLFIntegrator_swiginit(self, _nonlininteg.new_VectorConvectionNLFIntegrator(*args))

    @staticmethod
    def GetRule(fe, T):
        r"""GetRule(FiniteElement fe, ElementTransformation T) -> IntegrationRule"""
        return _nonlininteg.VectorConvectionNLFIntegrator_GetRule(fe, T)
    GetRule = _swig_new_static_method(_nonlininteg.VectorConvectionNLFIntegrator_GetRule)

    def AssembleElementVector(self, el, trans, elfun, elvect):
        r"""AssembleElementVector(VectorConvectionNLFIntegrator self, FiniteElement el, ElementTransformation trans, Vector elfun, Vector elvect)"""
        return _nonlininteg.VectorConvectionNLFIntegrator_AssembleElementVector(self, el, trans, elfun, elvect)
    AssembleElementVector = _swig_new_instance_method(_nonlininteg.VectorConvectionNLFIntegrator_AssembleElementVector)

    def AssembleElementGrad(self, el, trans, elfun, elmat):
        r"""AssembleElementGrad(VectorConvectionNLFIntegrator self, FiniteElement el, ElementTransformation trans, Vector elfun, DenseMatrix elmat)"""
        return _nonlininteg.VectorConvectionNLFIntegrator_AssembleElementGrad(self, el, trans, elfun, elmat)
    AssembleElementGrad = _swig_new_instance_method(_nonlininteg.VectorConvectionNLFIntegrator_AssembleElementGrad)

    def AssemblePA(self, *args):
        r"""
        AssemblePA(VectorConvectionNLFIntegrator self, FiniteElementSpace fes)
        AssemblePA(VectorConvectionNLFIntegrator self, FiniteElementSpace trial_fes, FiniteElementSpace test_fes)
        AssemblePA(VectorConvectionNLFIntegrator self, FiniteElementSpace fes)
        """
        return _nonlininteg.VectorConvectionNLFIntegrator_AssemblePA(self, *args)
    AssemblePA = _swig_new_instance_method(_nonlininteg.VectorConvectionNLFIntegrator_AssemblePA)

    def AddMultPA(self, x, y):
        r"""AddMultPA(VectorConvectionNLFIntegrator self, Vector x, Vector y)"""
        return _nonlininteg.VectorConvectionNLFIntegrator_AddMultPA(self, x, y)
    AddMultPA = _swig_new_instance_method(_nonlininteg.VectorConvectionNLFIntegrator_AddMultPA)
    __swig_destroy__ = _nonlininteg.delete_VectorConvectionNLFIntegrator

# Register VectorConvectionNLFIntegrator in _nonlininteg:
_nonlininteg.VectorConvectionNLFIntegrator_swigregister(VectorConvectionNLFIntegrator)

def VectorConvectionNLFIntegrator_GetRule(fe, T):
    r"""VectorConvectionNLFIntegrator_GetRule(FiniteElement fe, ElementTransformation T) -> IntegrationRule"""
    return _nonlininteg.VectorConvectionNLFIntegrator_GetRule(fe, T)
VectorConvectionNLFIntegrator_GetRule = _nonlininteg.VectorConvectionNLFIntegrator_GetRule



